/*
 * Copyright (C) 2022 favdb
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.exim.exporter;

import i18n.I18N;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import javax.swing.JOptionPane;
import storybook.exim.EXIM;
import storybook.toolkit.LOG;
import storybook.toolkit.StringUtil;
import storybook.ui.MainFrame;
import storybook.ui.dialog.ExceptionDlg;

/**
 * export proofreading observations
 *
 * @author favdb
 */
public class ExportReread extends AbstractExport {

	private static final String EXPORT_TITLE = "Proofread Export";

	public ExportReread(MainFrame mainFrame) {
		super(mainFrame, "html");
	}

	public static boolean exec(MainFrame mainFrame) {
		LOG.trace("ExportReread.exec(mainFrame)");
		ExportReread exp = new ExportReread(mainFrame);
		if (exp.openFile(I18N.getMsg("reread"))) {
			String str = exp.getReread();
			try {
				exp.outStream.write(str, 0, str.length());
				exp.outStream.flush();
			} catch (IOException ex) {
				ExceptionDlg.show(EXPORT_TITLE, ex);
				return false;
			}
		}
		exp.closeFile(true);
		return true;
	}

	@Override
	public boolean openFile(String name) {
		this.name = StringUtil.capitalize(I18N.getMsg("reread"));
		isOpened = false;
		param.setFileName(param.getDirectory()
				+ File.separator
				+ name + ".html");
		File file = new File(param.getDirectory());
		if (!(file.exists() && file.isDirectory())) {
			JOptionPane.showMessageDialog(mainFrame,
					I18N.getMsg("export.dir.error"),
					I18N.getMsg("export"), 1);
			return false;
		}
		if (!EXIM.askExists(null, param.getFileName())) {
			return false;
		}
		try {
			outStream = new BufferedWriter(new FileWriter(param.getFileName()));
		} catch (IOException ex) {
			ExceptionDlg.show(EXPORT_TITLE, ex);
			return (false);
		}
		isOpened = true;
		writeHeaderHtml();
		return true;
	}

	/**
	 * get all reread datas as HTML String
	 *
	 * @return
	 */
	private String getReread() {
		StringBuilder buf = new StringBuilder();
		return buf.toString();
	}

}
