/*
Storybook: Open Source software for novelists and authors.
Copyright (C) 2008 - 2012 Martin Mustun

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.model.hbn.entity;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import org.w3c.dom.Node;
import storybook.model.book.Book;
import storybook.model.hbn.entity.DB_DATA.DATA;

/**
 * Part generated by hbm2java
 *
 * @hibernate.class table="PART"
 */
@SuppressWarnings("serial")
public class Part extends AbstractEntity {

	public static Part findNumber(List<Part> parts, int n) {
		for (Part p : parts) {
			if (p.getNumber() == n) {
				return (p);
			}
		}
		return (null);
	}

	public static Integer getNextNumber(List<Part> parts) {
		int n = 0;
		for (Part p : parts) {
			if (p.getNumber() > n) {
				n = p.getNumber();
			}
		}
		return (n + 1);
	}

	private Integer number = 0;
	private Part superpart = null;
	private Timestamp creationTime = null;
	private Timestamp objectiveTime = null;
	private Timestamp doneTime = null;
	private Integer objectiveChars = 0;

	public Part() {
		super(Book.TYPE.PART, "111");
		this.creationTime = new Timestamp(new Date().getTime());
		setNotes("");
		setDescription("");
		setAssistant("");
	}

	public Part(Integer number, String name) {
		this();
		this.number = number;
		setName(name);
		this.objectiveChars = 0;
	}

	public Part(Integer number, String name, String notes, Part superpart,
			Timestamp creationTime, Timestamp objectiveTime,
			Timestamp doneTime) {
		this(number, name);
		setNotes(notes);
		this.superpart = superpart;
		this.creationTime = creationTime;
		this.objectiveTime = objectiveTime;
		this.doneTime = doneTime;
	}

	public String getIdent() {
		return String.format("%02d", getId());
	}

	public Integer getNumber() {
		return this.number;
	}

	public void setNumber(Integer number) {
		this.number = number;
	}

	public String getNumberName() {
		return this.number + ": " + this.getName();
	}

	public Part getSuperpart() {
		return this.superpart;
		//return(null);
	}

	public boolean hasSuperpart() {
		return superpart != null;
	}

	public void setSuperpart(Part superpart) {
		this.superpart = superpart;
	}

	public boolean isPartOfPart(Part ancestor) {
		return this.getId().equals(ancestor.getId());
	}

	public boolean hasCreationTime() {
		return creationTime != null;
	}

	public void setCreationTime(Timestamp ts) {
		creationTime = ts;
	}

	public Timestamp getCreationTime() {
		return creationTime;
	}

	public boolean hasObjectiveTime() {
		return objectiveTime != null;
	}

	public void setObjectiveTime(Timestamp ts) {
		objectiveTime = ts;
	}

	public Timestamp getObjectiveTime() {
		return objectiveTime;
	}

	public boolean hasObjectiveChars() {
		return objectiveChars != null;
	}

	public Integer getObjectiveChars() {
		return (hasObjectiveChars()) ? this.objectiveChars : 0;
	}

	public void setObjectiveChars(Integer objectiveChars) {
		this.objectiveChars = objectiveChars;
	}

	public boolean isDone() {
		return hasDoneTime();
	}

	public boolean hasDoneTime() {
		return doneTime != null;
	}

	public void setDoneTime(Timestamp ts) {
		doneTime = ts;
	}

	public Timestamp getDoneTime() {
		return doneTime;
	}

	@Override
	public String toString() {
		return getNumber() + ": " + getName() + (hasNotes() ? "*" : "");
	}

	@Override
	public String toCsv(String quoteStart, String quoteEnd, String separator) {
		StringBuilder b = new StringBuilder();
		b.append(quoteStart).append(getClean(this)).append(quoteEnd).append(separator);
		b.append(quoteStart).append(getClean(number)).append(quoteEnd).append(separator);
		b.append(quoteStart).append(getClean(getName())).append(quoteEnd).append(separator);
		b.append(quoteStart).append(getClean(getSuperpart())).append(quoteEnd).append(separator);
		b.append(quoteStart).append(getClean(getCreationTime())).append(quoteEnd).append(separator);
		b.append(quoteStart).append(getClean(getObjectiveTime())).append(quoteEnd).append(separator);
		b.append(quoteStart).append(getClean(getDoneTime())).append(quoteEnd).append(separator);
		b.append(quoteStart).append(getClean(getObjectiveChars())).append(quoteEnd).append(separator);
		b.append(quoteStart).append(getClean(getNotes())).append(quoteEnd).append(separator);
		return (b.toString());
	}

	@Override
	public String toHtml() {
		return (toCsv("<td>", "</td>", "\n"));
	}

	@Override
	public String toText() {
		return (toCsv("", "", "\t"));
	}

	@Override
	public String toDetail(boolean detailed) {
		StringBuilder b = new StringBuilder();
		b.append(toHtmlDetailHeader(detailed));
		b.append(getInfo("number", getNumber().toString()));
		if (getSuperpart() != null) {
			b.append(getInfo("sup", getClean(getSuperpart())));
		}
		if (getObjectiveTime() != null) {
			b.append(getInfo(DATA.OBJECTIVE_DATE, getClean(getObjectiveTime())));
			b.append(getInfo(DATA.OBJECTIVE_DONE, getClean(getDoneTime())));
			b.append(getInfo(DATA.OBJECTIVE_SIZE, getClean(getObjectiveChars())));
		} else if (detailed) {
			b.append(getInfo(DATA.OBJECTIVE_DATE, ""));
			b.append(getInfo(DATA.OBJECTIVE_DONE, ""));
			b.append(getInfo(DATA.OBJECTIVE_SIZE, ""));
		}
		b.append(toHtmlDetailFooter(detailed));
		return (b.toString());
	}

	@Override
	public String toXml() {
		StringBuilder b = new StringBuilder();
		b.append(toXmlTab(1)).append("<part \n");
		b.append(xmlCommon());
		b.append(xmlAttribute("name", getName()));
		b.append(xmlAttribute("number", getNumber().toString()));
		if (getSuperpart() != null) {
			b.append(xmlAttribute("sup", getClean(getSuperpart().getId())));
		}
		b.append(xmlAttribute("objective", getClean(getObjectiveTime())));
		b.append(xmlAttribute("done", getClean(getDoneTime())));
		b.append(xmlAttribute("chars", getClean(getObjectiveChars())));
		b.append(toXmlTab(2)).append(">\n");
		b.append(toXmlEnd());
		return (b.toString());
	}

	public static Part fromXml(Node node) {
		Part p = new Part();
		p.setId(getXmlLong(node, "id"));
		p.setNumber(getXmlInteger(node, "number"));
		p.setName(getXmlString(node, "name"));
		p.setDescription(getXmlText(node, "description"));
		p.setNotes(getXmlText(node, "notes"));
		p.setAssistant(getXmlText(node, "assistant"));
		return (p);
	}

	@Override
	public int compareTo(AbstractEntity ch) {
		return getNumber().compareTo(((Part) ch).getNumber());
	}

	@Override
	@SuppressWarnings("EqualsWhichDoesntCheckParameterClass")
	public boolean equals(Object obj) {
		if (!super.equals(obj)) {
			return false;
		}
		Part test = (Part) obj;
		boolean ret = true;
		ret = ret && equalsIntegerNullValue(number, test.getNumber());
		ret = ret && equalsStringNullValue(getName(), test.getName());
		ret = ret && equalsStringNullValue(getNotes(), test.getNotes());
		ret = ret && equalsObjectNullValue(superpart, test.getSuperpart());
		return ret;
	}

	@Override
	public int hashCode() {
		return hashPlus(super.hashCode(),
				number,
				superpart,
				creationTime,
				objectiveChars,
				objectiveTime,
				doneTime);
	}

	public static Part find(List<Part> list, String str) {
		for (Part elem : list) {
			if (elem.getName().equals(str)) {
				return (elem);
			}
		}
		return (null);
	}

	public static Part find(List<Part> list, Long id) {
		for (Part elem : list) {
			if (elem.id.equals(id)) {
				return (elem);
			}
		}
		return (null);
	}

	public static List<String> getDefColumns() {
		List<String> list = AbstractEntity.getDefColumns(Book.TYPE.PART);
		return (list);
	}

	public static List<String> getTable() {
		List<String> ls = new ArrayList<>();
		String tableName = "part";
		AbstractEntity.getTable(tableName, ls);
		ls.add(tableName + ",number,Integer,0");
		ls.add(tableName + ",part_id,Integer,0");
		ls.add(tableName + ",creation_ts,Time,0");
		ls.add(tableName + ",objective_ts,Time,0");
		ls.add(tableName + ",done_ts,Time,0");
		ls.add(tableName + ",objective_ch,Integer,0");
		return (ls);
	}

}
