/*
Storybook: Scene-based software for novelists and authors.
Copyright (C) 2008 - 2011 Martin Mustun

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.toolkit.net;

import i18n.I18N;
import java.awt.Desktop;
import java.io.File;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;
import javax.swing.JEditorPane;
import javax.swing.JOptionPane;
import javax.swing.event.HyperlinkEvent;
import storybook.model.hbn.entity.Location;
import storybook.toolkit.LOG;
import storybook.toolkit.ListUtil;

public class Net {

	public enum URL {
		HOME("http://ostorybook.tuxfamily.org/v5/"),
		CONTACT(HOME + "fortopic.php"),
		DOC(HOME + "articles.php?pg=41&mnuid=2&tconfig=2"),
		FAQ(HOME + "faq.php"),
		REPORTBUG("http://ostorybook.tuxfamily.org/" + "support"),
		DOWNLOAD("http://download.tuxfamily.org/ostorybook/"),
		DOWNLOAD_CURRENT(DOWNLOAD + "current/"),
		VERSION_TXT(DOWNLOAD_CURRENT + "Versions.txt"),
		UPDATE(DOWNLOAD_CURRENT + "Update.zip"),
		SPELL_REMOTE_DIR(DOWNLOAD + "dictionary/"),
		ASSISTANT_REMOTE_DIR(DOWNLOAD + "Assistant/"),
		OSM("http://www.openstreetmap.org/"),
		DO_UPDATE("false");
		private final String text;

		private URL(String text) {
			this.text = text;
		}

		@Override
		public String toString() {
			return text;
		}
	}

	public static void openBrowser(String path) {
		//App.trace("Net.openBrowser(" + path + ")");
		try {
			Desktop.getDesktop().browse(new URI(path));
		} catch (URISyntaxException | IOException e) {
			LOG.err("Net.openBrowser(" + path + ")", e);
		}
	}

	public static void openOSM(Location location) {
		//App.trace("Net.openOSM(location=" + location.toString() + ")");
		List<String> list = new ArrayList<>();
		if (!location.getCity().isEmpty()) {
			list.add(location.getCity());
			if (!location.getAddress().isEmpty()) {
				list.add(location.getAddress());
			}
			if (!location.getCountry().isEmpty()) {
				list.add(location.getCountry());
			}
		}
		String b = ListUtil.join(list);
		String str = "";
		try {
			str = "search?query=" + URLEncoder.encode(b, "UTF-8");
		} catch (UnsupportedEncodingException ex) {
			LOG.err("URMEncoder", ex);
		}
		if (location.getGps() != null && !location.getGps().isEmpty()) {
			str = "#map=15/" + location.getGps();
		} else if (str.isEmpty()) {
			JOptionPane.showMessageDialog(null,
					I18N.getMsg("location.gps.no_position"),
					I18N.getMsg("location.gps.show"),
					JOptionPane.ERROR_MESSAGE);
			return;
		}
		openOSM(str);
	}

	public static void openOSM(String query) {
		//App.trace("Net.openOSM(query=" + query + ")");
		String path = URL.OSM.toString() + query;
		openBrowser(path);
	}

	/**
	 * open an URL, may be a local file
	 *
	 * @param e
	 */
	public static void openUrl(HyperlinkEvent e) {
		//App.trace("Net.openUrl(description=\"" + e.getDescription() + "\")");
		String url = e.getDescription();
		try {
			if (url.toLowerCase().startsWith("http")) {
				//App.trace("open browser");
				Desktop.getDesktop().browse(new URI(url));
			} else if (url.startsWith("#")) {
				//App.trace("internal link");
				if (e.getSource() instanceof JEditorPane) {
					//App.trace("yes it is");
					JEditorPane ep = (JEditorPane) e.getSource();
					ep.scrollToReference(url.substring(1));
				}
			} else if (url.startsWith("file")) {
				openFile(url);
			} else {
				// trying to open any other link by default application
				Desktop.getDesktop().browse(new URI(url));
			}
		} catch (URISyntaxException ex) {
			LOG.err("Net.openUrl URI exception ", ex);
		} catch (IOException ex) {
			LOG.err("Net.openUrl IOException for " + url, ex);
		}
	}

	private static void openFile(String url) {
		//App.trace("Net.openFile(url=\"" + url + "\")");
		File file = new File(url.replace("file://", ""));
		if (file.exists()) {
			try {
				Desktop.getDesktop().edit(file);
			} catch (UnsupportedOperationException ex) {
				try {
					Desktop.getDesktop().open(file);
				} catch (IOException ex1) {
					LOG.err("file exception for open " + url);
				}
			} catch (IOException ex) {
				LOG.err("file exception for " + url);
			}
		} else {
			LOG.err("warning file not exists: " + file.getAbsolutePath());
		}
	}

}
