/*
Storybook: Open Source software for novelists and authors.
Copyright (C) 2008 - 2012 Martin Mustun

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.ui.table;

import i18n.I18N;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.PropertyChangeEvent;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JToolBar;
import org.hibernate.Session;
import resources.icons.ICONS;
import storybook.App;
import storybook.ctrl.ActKey;
import storybook.ctrl.Ctrl.PROPS;
import storybook.model.EntityUtil;
import storybook.model.Model;
import storybook.model.book.Book;
import storybook.model.hbn.dao.SceneDAOImpl;
import storybook.model.hbn.entity.AbstractEntity;
import storybook.model.hbn.entity.Chapter;
import storybook.model.hbn.entity.DB_DATA;
import storybook.model.hbn.entity.Part;
import storybook.model.hbn.entity.Person;
import storybook.model.hbn.entity.Scene;
import storybook.model.hbn.entity.Status;
import storybook.model.hbn.entity.Status.STATUS;
import storybook.model.hbn.entity.Strand;
import storybook.model.state.SceneStatus;
import storybook.toolkit.DateUtil;
import storybook.toolkit.SbDuration;
import storybook.toolkit.TextUtil;
import storybook.toolkit.comparator.ObjectComparator;
import storybook.toolkit.html.Html;
import storybook.ui.MainFrame;
import storybook.ui.Ui;
import storybook.ui.renderer.lcr.LCRStatus;
import static storybook.ui.table.AbsColumn.*;

/**
 * @author martin
 *
 */
public class SceneTable extends AbstractTable implements ActionListener {

	private static final String TT = "SceneTable";

	public enum ACT {
		CB_STATUS,
		CB_STRAND,
		CB_NARRATOR,
		NONE;
	}

	public ACT getACT(String act) {
		for (ACT a : ACT.values()) {
			if (a.name().equals(act)) {
				return (a);
			}
		}
		return (ACT.NONE);
	}

	private static final String BT_EXTERNAL = "BtEXTERNAL";

	private JComboBox cbStatus;
	private JComboBox cbStrand;
	private JComboBox cbNarrator;

	public SceneTable(MainFrame mainFrame) {
		super(mainFrame, Book.TYPE.SCENE);
	}

	@Override
	public void init() {
		withPart = true;
	}

	@SuppressWarnings({"unchecked"})
	@Override
	public List<AbstractEntity> getAllEntities() {
		//LOG.trace(TT + ".getAllEntities()");
		int status = cbStatus.getSelectedIndex();
		Part part = null;
		if (cbPartFilter != null && cbPartFilter.getSelectedIndex() > 0) {
			part = (Part) cbPartFilter.getSelectedItem();
		}
		Strand strand = null;
		if (cbStrand.getSelectedIndex() > 0) {
			strand = (Strand) cbStrand.getSelectedItem();
		}
		Person narrator = null;
		if (cbNarrator.getSelectedIndex() > 0) {
			narrator = (Person) cbNarrator.getSelectedItem();
		}
		List<AbstractEntity> entities = EntityUtil.findEntities(mainFrame, getType());
		List<AbstractEntity> scenes = new ArrayList<>();
		for (AbstractEntity e : entities) {
			Scene s = (Scene) e;
			if (part != null && s.hasChapter()) {
				Chapter chapter = s.getChapter();
				if (chapter.hasPart() && !chapter.getPart().equals(part)) {
					continue;
				}
			}
			boolean b = true;
			if (status < STATUS.values().length && s.getStatus() != status) {
				b = false;
			}
			if (cbNarrator.getSelectedIndex() > 0 && narrator != null && !narrator.equals(s.getNarrator())) {
				b = false;
			}
			if (cbStrand.getSelectedIndex() > 0) {
				if (strand != null && !strand.equals(s.getStrand())
						&& (s.getStrands() != null && !s.getStrands().contains(strand))) {
					b = false;
				}
			}
			if (b) {
				scenes.add(s);
			}
		}
		return (scenes);
	}

	@Override
	@SuppressWarnings("unchecked")
	public JToolBar initToolbar() {
		//LOG.trace(TT + ".initToolbar(withPart=" + (withPart ? "true" : "false") + ")");
		super.initToolbar();
		toolbar.add(new JLabel(I18N.getColonMsg("status")));
		cbStatus = initCbStatus();
		toolbar.add(cbStatus);

		toolbar.add(new JLabel(I18N.getColonMsg("strand")));
		cbStrand = new JComboBox();
		cbStrand.setName(ACT.CB_STRAND.toString());
		Ui.fillComboBox(cbStrand, EntityUtil.findEntities(mainFrame, Book.TYPE.STRAND), !EMPTY, ALL, this);
		cbStrand.setSelectedIndex(0);
		cbStrand.addActionListener(this);
		toolbar.add(cbStrand);

		toolbar.add(new JLabel(I18N.getColonMsg("scene.narrator")));
		cbNarrator = new JComboBox();
		cbNarrator.setName(ACT.CB_NARRATOR.toString());
		Ui.fillComboBox(cbNarrator, SceneDAOImpl.findNarrators(mainFrame), !EMPTY, ALL, this);
		cbNarrator.setSelectedIndex(0);
		cbNarrator.addActionListener(this);
		toolbar.add(cbNarrator);
		return (toolbar);
	}

	@SuppressWarnings("unchecked")
	private JComboBox initCbStatus() {
		JComboBox cb = new JComboBox();
		cb.setName("cbStatus");
		for (Status.STATUS st : Status.STATUS.values()) {
			cb.addItem(st.ordinal());
		}
		cb.addItem(I18N.getMsg("status.all"));
		cb.setSelectedIndex(cb.getItemCount() - 1);
		cb.setRenderer(new LCRStatus());
		cb.addActionListener(this);
		return (cb);
	}

	/**
	 * initialize the footer
	 *
	 * @return
	 */
	@Override
	public JToolBar initFooter() {
		JToolBar footer = super.initFooter();
		if (mainFrame.getBook().isUseXeditor()) {
			btExternal = Ui.initButton(BT_EXTERNAL, book.getParamEditor().getName(),
					ICONS.K.NONE, "", e -> sendSetExternal(table.getSelectedRow()));
			btExternal.setText(book.getParamEditor().getName());
			btExternal.setToolTipText(I18N.getMsg("xeditor.launching")
					+ " " + book.getParamEditor().getName());
			btExternal.setEnabled(false);
			footer.add(btExternal);
		}
		return footer;
	}

	@Override
	public synchronized void actionPerformed(ActionEvent e) {
		//LOG.trace(TT + ".actionPerformed(" + e.toString() + ")");
		if (e.getSource() instanceof JComboBox) {
			fillTable();
			return;
		}
		super.actionPerformed(e);
	}

	@Override
	protected void modelPropertyChangeLocal(PropertyChangeEvent evt) {
		//LOG.trace(TT + ".modelPropertyChangeLocal(evt) propName=" + evt.getPropertyName());
		try {
			String propName = evt.getPropertyName();
			Object newValue = evt.getNewValue();
			ActKey act = new ActKey(evt);
			if (isInit(act)) {
				return;
			}
			switch (Book.getTYPE(act.type)) {
				case PART:
					if (isChange(act)) {
						Ui.fillComboBox(cbPartFilter, EntityUtil.findEntities(mainFrame, getType()), !EMPTY, !ALL, this);
					}
					break;
				case SCENE:
					if (PROPS.SCENE_FILTER_STATUS.check(propName)) {
						initTableModel(evt);
						if (newValue instanceof SceneStatus) {
							cbStatus.setSelectedItem((SceneStatus) newValue);
						}
					} else if (PROPS.SCENE_FILTER_NARRATOR.check(propName)) {
						initTableModel(evt);
						if (newValue instanceof String) {
							cbNarrator.setSelectedItem((String) newValue);
						}
					} else if (PROPS.SCENE_FILTER_STRAND.check(propName)) {
						initTableModel(evt);
						if (newValue instanceof String) {
							cbStrand.setSelectedItem((String) newValue);
						}
					}
					break;
				case STRAND:
					if (isChange(act)) {
						int nx = cbStrand.getSelectedIndex();
						Strand os = (Strand) cbStrand.getSelectedItem();
						Ui.fillComboBox(cbStrand, EntityUtil.findEntities(mainFrame, Book.TYPE.STRAND), !EMPTY, !ALL);
						if (nx != 0 && os != null) {
							cbStrand.setSelectedItem(os);
						}
					}
					break;
				default:
					break;
			}
		} catch (Exception e) {
		}
	}

	@Override
	protected AbstractEntity getEntity(Long id) {
		Model model = mainFrame.getBookModel();
		Session session = model.beginTransaction();
		SceneDAOImpl dao = new SceneDAOImpl(session);
		Scene scene = dao.find(id);
		model.commit();
		return scene;
	}

	@Override
	public void updateRow(AbstractEntity entity) {
		// refresh for chapter, item, location, person, scenario, scene, strand, photo
		switch (Book.getTYPE(entity)) {
			case CHAPTER:
			case ITEM:
			case LOCATION:
			case PLOT:
			//case SCENARIO:
			case SCENE:
				fillTable();
				break;
			case STRAND:
				Ui.fillComboBox(cbStrand, EntityUtil.findEntities(mainFrame, Book.TYPE.STRAND), !EMPTY, ALL, this);
				fillTable();
				break;
			case PERSON:
				Ui.fillComboBox(cbNarrator, SceneDAOImpl.findNarrators(mainFrame), !EMPTY, ALL, this);
				fillTable();
			default:
				break;
		}
	}

	@Override
	public List<AbsColumn> getColumns(AbstractEntity entity) {
		List<AbsColumn> cols = super.getColumns(entity);
		AbsColumn col;
		cols.add(new AbsColumn(mainFrame, cols, DB_DATA.DATA.INFORMATIVE, TCR_HIDE, AL_CENTER, TCR_BOOLEAN));
		cols.add(new AbsColumn(mainFrame, cols, DB_DATA.DATA.NUMBER, NUMERIC, TCR_HIDE, AL_CENTER));
		cols.add(new AbsColumn(mainFrame, cols, DB_DATA.DATA.STATUS, AL_CENTER, TCR_STATUS));
		cols.add(new AbsColumn(mainFrame, cols, DB_DATA.DATA.INTENSITY, AL_CENTER, TCR_HIDE, TCR_COLOR, AL_CENTER));
		cols.add(new AbsColumn(mainFrame, cols, DB_DATA.DATA.CCSS, TCR_HIDE, AL_CENTER));
		col = new AbsColumn(mainFrame, cols, DB_DATA.DATA.SCENE_CHAPTER, TCR_ENTITY);
		col.setComparator(new ObjectComparator());
		cols.add(col);
		cols.add(new AbsColumn(mainFrame, cols, DB_DATA.DATA.WORDS, NUMERIC_RENDERER, AL_CENTER));
		col = new AbsColumn(mainFrame, cols, DB_DATA.DATA.SCENE_SCENETS, TCR_DATE, TCR_HIDE, AL_CENTER);
		col.setMsgKey("date");
		col.setShowDateTime(true);
		cols.add(col);
		col = new AbsColumn(mainFrame, cols, DB_DATA.DATA.SCENE_RELATIVESCENEID, TCR_HIDE, AL_CENTER, TCR_SCENEID);
		col.setMsgKey("scene.relativedate.after");
		cols.add(col);
		cols.add(new AbsColumn(mainFrame, cols, DB_DATA.DATA.SCENE_RELATIVETIME, TCR_HIDE, AL_CENTER));
		cols.add(new AbsColumn(mainFrame, cols, DB_DATA.DATA.DURATION, TCR_HIDE, AL_CENTER));
		cols.add(new AbsColumn(mainFrame, cols, DB_DATA.DATA.SCENE_STRAND, TCR_ENTITY, AL_CENTER));
		cols.add(new AbsColumn(mainFrame, cols, DB_DATA.DATA.STRANDS, TCR_HIDE, TCR_ENTITIES, AL_CENTER));
		cols.add(new AbsColumn(mainFrame, cols, DB_DATA.DATA.SCENE_NARRATOR, TCR_HIDE, AL_CENTER, TCR_ENTITY));
		cols.add(new AbsColumn(mainFrame, cols, DB_DATA.DATA.SCENE_SUMMARY, TCR_HIDE));
		cols.add(new AbsColumn(mainFrame, cols, DB_DATA.DATA.PERSONS, TCR_HIDE, AL_CENTER, TCR_ENTITIES));
		cols.add(new AbsColumn(mainFrame, cols, DB_DATA.DATA.LOCATIONS, TCR_HIDE, AL_CENTER, TCR_ENTITIES));
		cols.add(new AbsColumn(mainFrame, cols, DB_DATA.DATA.ITEMS, TCR_HIDE, AL_CENTER, TCR_ENTITIES));
		cols.add(new AbsColumn(mainFrame, cols, DB_DATA.DATA.PLOTS, TCR_HIDE, AL_CENTER, TCR_ENTITIES));
		if (book.isScenario()) {
			cols.add(new AbsColumn(mainFrame, cols, DB_DATA.DATA.SCENARIO_PITCH, TCR_HIDE));
			cols.add(new AbsColumn(mainFrame, cols, DB_DATA.DATA.SCENARIO_STAGE, TCR_HIDE));
			cols.add(new AbsColumn(mainFrame, cols, DB_DATA.DATA.SCENARIO_LOC, AL_CENTER, TCR_HIDE));
			cols.add(new AbsColumn(mainFrame, cols, DB_DATA.DATA.SCENARIO_MOMENT, AL_CENTER, TCR_HIDE));
			cols.add(new AbsColumn(mainFrame, cols, DB_DATA.DATA.SCENARIO_START, AL_CENTER, TCR_HIDE));
			cols.add(new AbsColumn(mainFrame, cols, DB_DATA.DATA.SCENARIO_END, AL_CENTER, TCR_HIDE));
		} else if (App.getAssistant().isExists("stage")) {
			cols.add(new AbsColumn(mainFrame, cols, DB_DATA.DATA.SCENARIO_STAGE, TCR_HIDE));
		}
		getColumnsEnd(cols, entity);
		return cols;
	}

	@Override
	public List<Object> getRow(AbstractEntity entity) {
		List<Object> cols = super.getRow(entity);
		Scene e = (Scene) entity;
		cols.add(e.getInformative());
		cols.add(e.getSceneno());
		cols.add(e.getStatus());
		cols.add(e.getIntensity());
		cols.add(e.getCCSS());
		cols.add(e.getChapter());
		cols.add(e.getWords());
		if (e.hasScenets()) {// fixed date
			cols.add(e.getDate());
			cols.add(-1L);
			cols.add(" ");
		} else if (e.hasRelativescene()) {// no fixed date compute the relative date
			Scene sx = (Scene) EntityUtil.findEntity(mainFrame, Book.TYPE.SCENE, e.getRelativesceneid());
			if (sx != null && sx.getDate() != null) {
				//LOG.trace("relative=" + e.getRelativetime());
				Date dx = DateUtil.addDateTime(sx.getDate(), e.getRelativetime());
				cols.add(dx);
			} else {
				cols.add(" ");
			}
			cols.add(e.getRelativesceneid());
			SbDuration dur;
			if (e.getRelativetime().isEmpty()) {
				dur = new SbDuration(SbDuration.computeFromWords(e.getSummary()));
			} else {
				dur = new SbDuration(e.getRelativetime());
			}
			cols.add(dur.toText(I18N.getMsg("duration.initiales")));
		} else {
			cols.add("");
			cols.add(-1L);
			cols.add(" ");
		}
		cols.add(e.getDurationToText());
		cols.add(e.getStrand());
		cols.add(e.getStrands());
		cols.add(e.getNarrator());
		cols.add(TextUtil.ellipsize(Html.htmlToText(e.getSummary()), 128));
		cols.add(e.getPersons());
		cols.add(e.getLocations());
		cols.add(e.getItems());
		cols.add(e.getPlots());
		// scenario data
		if (book.isScenario()) {
			cols.add(e.getScenariopitch());
			cols.add(e.getScenariostage());
			cols.add(e.getScenarioloc());
			cols.add(e.getScenariomoment());
			cols.add(e.getScenariostart());
			cols.add(e.getScenarioend());
		} else if (App.getAssistant().isExists("stage")) {
			cols.add(e.getScenariostage());
		}
		getRowEnd(cols, entity);
		return cols;
	}

}
